import { useMutation, useQuery, useQueryClient } from "@tanstack/react-query";
import { useNavigate } from "@tanstack/react-router";
import { useEffect, useState } from "react";

import { handleServerError } from "@/utils/handle-server-error";
import { ApiError } from "@/errors/api-error";
import {
	DashboardService,
	LoginService,
	ShopLoginAccessTokenData,
	UserPublic,
	UserRegister,
	UserService,
	UserUpdate
} from "@/client";
import { toast } from "./useToast";

const isLoggedIn = () => {
	return localStorage.getItem("access_token") !== null;
};

const useAuth = () => {
	const [error, setError] = useState<string | null>(null);
	const [loggedIn, setLoggedIn] = useState(isLoggedIn());
	const navigate = useNavigate();
	const queryClient = useQueryClient();
	const { data: user } = useQuery<UserPublic | null, Error>({
		queryKey: ["currentUser"],
		queryFn: DashboardService.userGetUser,
		enabled: loggedIn
	});

	const signUpMutation = useMutation({
		mutationFn: (data: UserRegister) =>
			DashboardService.userRegister({ requestBody: data }),
		onSuccess: () => navigate({ to: "/sign-in" }),
		onError: (err: ApiError) => handleServerError(err),
		onSettled: () => queryClient.invalidateQueries({ queryKey: ["users"] })
	});

	const login = async (data: ShopLoginAccessTokenData) => {
		const response = await LoginService.dashboardLoginAccessToken({
			formData: {
				username: data.formData.username,
				password: data.formData.password
			}
		});
		localStorage.setItem("access_token", response.access_token);
		setLoggedIn(true);
		await queryClient.invalidateQueries({ queryKey: ["currentUser"] });
	};

	const loginMutation = useMutation({
		mutationFn: login,
		onSuccess: () => navigate({ to: "/" }),
		onError: (err: ApiError) => handleServerError(err)
	});

	const logout = () => {
		localStorage.removeItem("access_token");
		setLoggedIn(false);
		queryClient.invalidateQueries({ queryKey: ["currentUser"] });
		navigate({ to: "/", replace: true, reloadDocument: true });
	};

	const updateAccountMutation = useMutation({
		mutationFn: (data: UserUpdate) =>
			UserService.userUpdateUser({ requestBody: data }),
		onSuccess: () => {
			toast({ title: "Account updated successfully" });
			queryClient.invalidateQueries({ queryKey: ["currentUser"] });
		},
		onError: (err: ApiError) => handleServerError(err)
	});

	useEffect(() => {
		const handleStorageChange = (event: StorageEvent) => {
			if (event.key === "access_token") {
				const tokenExists = event.newValue !== null;
				setLoggedIn(tokenExists);
				if (!tokenExists) {
					queryClient.removeQueries({ queryKey: ["currentUser"] });
				} else {
					queryClient.invalidateQueries({ queryKey: ["currentUser"] });
				}
			}
		};

		window.addEventListener("storage", handleStorageChange);
		return () => window.removeEventListener("storage", handleStorageChange);
	}, [queryClient]);

	return {
		signUpMutation,
		loginMutation,
		updateAccountMutation,
		logout,
		user,
		error,
		resetError: () => setError(null)
	};
};

export { isLoggedIn };
export default useAuth;
